<?php

/*
 * AfterLogic Admin Panel by AfterLogic Corp. <support@afterlogic.com>
 *
 * Copyright (C) 2002-2009  AfterLogic Corp. (www.afterlogic.com)
 * Distributed under the terms of the license described in LICENSE.txt
 * 
 */

class ImapStorage extends MailServerStorage
{
	/**
	 * @access private
	 * @var IMAPMAIL
	 */
	var $_imapMail;

	/**
	 * @param Account $account
	 * @return ImapStorage
	 */
	function ImapStorage(&$account, $pathToClassFolder)
	{
		MailServerStorage::MailServerStorage($account);
		require_once($pathToClassFolder.'libs/class_imap.php');
		require_once($pathToClassFolder.'class_folders.php');
		
		$this->_imapMail = new IMAPMAIL();
		$this->_imapMail->host = $account->MailIncHost;
		$this->_imapMail->port = $account->MailIncPort;
		$this->_imapMail->user = $account->MailIncLogin;
		$this->_imapMail->password = $account->MailIncPassword;
	}

	/**
	 * @return bool
	 */
	function Connect()
	{
		if($this->_imapMail->connection != false)
		{
			return true;
		}
		
		if (!$this->_imapMail->open())
		{
			$this->SetError(ap_Utils::TakePhrase('WM_ERROR_IMAP4_CONNECT'));
			return false;
		}
		else
		{
			register_shutdown_function(array(&$this, 'Disconnect'));	
		}
		
		if (!$this->_imapMail->login($this->_account->MailIncLogin, $this->_account->MailIncPassword))
		{
			$this->SetError(ap_Utils::TakePhrase('WM_ERROR_POP3IMAP4AUTH'));
			return false;				
		}

		return true;
	}

	/**
	 * @return bool
	 */
	function Disconnect()
	{
		if(!$this->_imapMail->connection)
		{
			return true;
		}
		return $this->_imapMail->close();
	}

	/**
	 * @return FolderCollection
	 */
	function GetFolders()
	{
		$folderCollection = new FolderCollection();
		$folders =& $this->_imapMail->list_mailbox($this->_account->Delimiter);
		$subs_folders = $this->_imapMail->list_subscribed_mailbox($this->_account->Delimiter);
		$this->_addLevelToFolderTree($folderCollection, $folders, $subs_folders);
		return $folderCollection;
	}

	/**
	 * @param Folder $folder
	 * @return bool
	 */
	function CreateFolder(&$folder)
	{
		if ($this->_imapMail->create_mailbox($folder->FullName))
		{
			if (!$folder->Hide)
			{
				$this->_imapMail->subscribe_mailbox($folder->FullName);
			}
			return true;
		}
		return false;
	}
	
	function SubscribeFolder(&$folder, $isHide = false)
	{
		if ($isHide)
		{
			$this->_imapMail->unsubscribe_mailbox($folder->FullName);
		}
		else
		{
			$this->_imapMail->subscribe_mailbox($folder->FullName);
		}
	}

	/**
	 * @param Folder $folder
	 * @return bool
	 */
	function DeleteFolder(&$folder)
	{
		return $this->_imapMail->delete_mailbox($folder->FullName);
	}

	/**
	 * @access private
	 * @param FolderCollection $folderCollection
	 * @param array $folders
	 * @param array $subsfolders
	 * @param string $rootPrefix optional
	 */
	function _addLevelToFolderTree(&$folderCollection, &$folders, $subsfolders, $rootPrefix = '', $foldersAdd = array(false, false, false, false), $isInbox = false)
	{
		$InboxAdd = $foldersAdd[0];
		$SentAdd = $foldersAdd[1];
		$DraftsAdd = $foldersAdd[2];
		$TrashAdd = $foldersAdd[3];
		
		$prefixLen = strlen($rootPrefix);
		$foldersCount = count($folders);
		for ($i = 0; $i < $foldersCount; $i++)
		{
			$folderFullName = $folders[$i];
			if ($rootPrefix != $folderFullName && strlen($folderFullName) > $prefixLen && substr($folderFullName, 0, $prefixLen) == $rootPrefix && strpos($folderFullName, $this->_account->Delimiter, $prefixLen + 1) === false)
			{
				$strLen = ($prefixLen == 0) ? $prefixLen : $prefixLen - 1;
				$name = trim(substr($folderFullName, $strLen), $this->_account->Delimiter);
				$names = explode($this->_account->Delimiter, $name);
				$folderObj = new Folder($this->_account->Id, -1, $folderFullName, $names[0]);
				
				if ($prefixLen == 0 || $isInbox)
				{
					switch ($folderObj->Type)
					{
						case WM_FOLDERTYPE_Inbox:
							if ($InboxAdd) $folderObj->Type = WM_FOLDERTYPE_Custom;
							$InboxAdd = true;
							break;
						case WM_FOLDERTYPE_SentItems:
							if ($SentAdd) $folderObj->Type = WM_FOLDERTYPE_Custom;
							$SentAdd = true;
							break;
						case WM_FOLDERTYPE_Drafts:
							if ($DraftsAdd) $folderObj->Type = WM_FOLDERTYPE_Custom;
							$DraftsAdd = true;
							break;
						case WM_FOLDERTYPE_Trash:
							if ($this->_account->_settings->Imap4DeleteLikePop3)
							{
								if ($TrashAdd) $folderObj->Type = WM_FOLDERTYPE_Custom;
								$TrashAdd = true;
							}
							else
							{
								$folderObj->Type = WM_FOLDERTYPE_Custom;
							}
							break;
						default:
							$folderObj->Type = WM_FOLDERTYPE_Custom;
							break;				
					}
				}
				else 
				{
					$folderObj->Type = WM_FOLDERTYPE_Custom;
				}
				
				$folderObj->Hide = (USE_LSUB) ? false : !in_array($folderObj->FullName, $subsfolders);
				if ($folderObj->Type != WM_FOLDERTYPE_Custom)
				{
					$folderObj->Hide = false;
				}
				
				$newCollection = new FolderCollection();
				if ($folderObj->Type == WM_FOLDERTYPE_Inbox)
				{
					$this->_addLevelToFolderTree($newCollection, $folders, $subsfolders, $folderFullName.$this->_account->Delimiter, 
						array($InboxAdd, $SentAdd, $DraftsAdd, $TrashAdd), true);	
				}
				else 
				{
					$this->_addLevelToFolderTree($newCollection, $folders, $subsfolders, $folderFullName.$this->_account->Delimiter,
						array($InboxAdd, $SentAdd, $DraftsAdd, $TrashAdd));	
				}
				
				if ($newCollection->Count() > 0)
				{
					$folderObj->SubFolders = $newCollection;
				}
				
				$folderCollection->Add($folderObj);
				
				unset($folderObj, $newCollection);
			}
		}
	}
}