<?php

/*
 * AfterLogic Admin Panel by AfterLogic Corp. <support@afterlogic.com>
 *
 * Copyright (C) 2002-2009  AfterLogic Corp. (www.afterlogic.com)
 * Distributed under the terms of the license described in LICENSE.txt
 * 
 */

class Xmail_TabConfig
{
	/**
	 * @var	array
	 */
	var $_lines = array();
	
	/**
	 * @var	string
	 */
	var $_tab = "\t";
	
	/**
	 * @var	string
	 */
	var $_crlf = "\r\n";
	
	/**
	 * Server Settings that must be inverted for interpretation
	 * @var	array
	 */
	var $_settingsWithInvertion = array(	
										'SmtpProto',
										'Pop3Proto',
										'Imap4Proto',
										'EnablePOP3-TLS',
										'FingerProto',
										'ControlProto',
										'EnableCTRL-TLS',
										'AllowSmtpETRN',
										'AllowSmtpVRFY',
										'EnableSMTP-TLS');
	
	/**
	 * @param	string	$raw[optional] = null
	 * @return	Xmail_TabConfig
	 */
	function Xmail_TabConfig($raw = null)
	{
		if ($raw !== null)
		{
			$lines = explode($this->_crlf, $raw);
			foreach ($lines as $line) 
			{
				$line = new Xmail_TabConfigLine($line);
				$this->AddLine($line);
				unset($line);
			}
		}
	}
	
	/**
	 * @param	Xmail_TabConfigLine	$line
	 */
	function AddLine($line)
	{
		
		$this->_lines[] =& $line;
	}
	
	/**
	 * @return	string
	 */
	function ToString()
	{
		$out = '';
		foreach ($this->_lines as $line)
		{
			$out .= $line->ToString().$this->_crlf;
		}
		return trim($out);
	}
	
	/**
	 * @return	array
	 */
	function ToArray()
	{
		$out = array();
		foreach ($this->_lines as $line)
		{
			$out[] = $line->ToString();
		}
		return $out;
	}
	
	/**
	 * @param	string	$key
	 * @return	string|null
	 */
	function GetValueByKey($key)
	{
		foreach ($this->_lines as $line)
		{
			if ($line->Key == $key)
			{
				return $line->Value;
			}
		}
		return null;
	}
	
	/**
	 * @param	string	$key
	 * @return	Xmail_TabConfigLine|null
	 */
	function &GetByKey($key)
	{
		$null = null;
		foreach ($this->_lines as $line)
		{
			if ($line->Key == $key)
			{
				return $line;
			}
		}
		return $null;
	}
	
	/**
	 * @param	string	$key
	 * @return	bool
	 */
	function IsSetValue($key)
	{
		foreach ($this->_lines as $line)
		{
			if ($line->Key == $key)
			{
				return true;
			}
		}
		return false;
	}
	
	/**
	 * @param	string	$key
	 * @return	bool
	 */
	function IsCommentValue($key)
	{
		foreach ($this->_lines as $line)
		{
			if ($line->Key == $key && $line->isCommentary)
			{
				return true;
			}
		}
		return false;
	}
	
	/**
	 * @param	string	$key
	 * @param	bool	$value[optional] = true
	 */
	function SetIsComment($key, $value = true)
	{
		foreach ($this->_lines as $line)
		{
			if ($line->Key == $key)
			{
				$line->isCommentary = $value;
			}
		}
	}

	/**
	 * @param	string	$key
	 * @param	string	$value
	 */
	function SetValueByKey($key, $value)
	{
		if ($value === null)
		{
			return false;
		}
		
		$exist = false;
		for ($i = 0, $c = count($this->_lines); $i < $c; $i++)
		{
			$line =& $this->_lines[$i];
			if ($line->Key == $key)
			{
				$line->Value = $value;
				$line->isCommentary = false;
				$exist = true;
			}
		}
		
		if (!$exist)
		{
			$this->_lines[] = new Xmail_TabConfigLine('"'.$key.'"'.$this->_tab.'"'.$value.'"');
		}
		
		return true;
	}
	
	function GetBoolValueByKey($key)
	{
		$line =& $this->GetByKey($key);
		if ($line !== null && !$line->isCommentary)
		{
			$return = (bool) $line->Value;
			if (in_array($key, $this->_settingsWithInvertion))
			{
				$return = !$return;
			}
			return $return;
		}
		
		return false;
	}
	
	/**
	 * @param	string	$key
	 * @param	bool	$value
	 */
	function SetBoolValueByKey($key, $value)
	{
		if (in_array($key, $this->_settingsWithInvertion))
		{
			$value = !$value;
		}
		$value = ($value) ? '1' : '0';
		$this->SetValueByKey($key, $value);
	}
}

class Xmail_TabConfigLine
{
	/**
	 * @var	bool
	 */
	var $isCommentary = false;
	
	/**
	 * @var	bool
	 */
	var $isEmptyLine = false;
	
	/**
	 * @var	string
	 */
	var $_line;
	
	/**
	 * @var	string
	 */
	var $Key;
	
	/**
	 * @var	string
	 */
	var $Value;
	
	/**
	 * @var	string
	 */
	var $_tab = "\t";
	
	/**
	 * @param	string	$line
	 * @return	Xmail_TabConfigLine
	 */
	function Xmail_TabConfigLine($line)
	{
		$this->_line =& $line;
		$this->_parse($this->_line);
	}

	/**
	 * @return	string
	 */
	function ToString()
	{
		if ($this->isCommentary || $this->isEmptyLine)
		{
			if (strlen($this->Key) > 0 && $this->isCommentary)
			{
				return '#"'.$this->Key.'"'.$this->_tab.'"'.$this->Value.'"';
			}
			else if (strlen($this->_line) > 0)
			{
				if ($this->_line{0} != '#' && $this->isCommentary)
				{
					$this->_line = '#'.$this->_line;
				}
				return $this->_line;
			}
		}
		else
		{
			if (strlen($this->Key) > 0)
			{
				return '"'.$this->Key.'"'.$this->_tab.'"'.$this->Value.'"';
			}
		}
		return '';
	}
	
	/**
	 * @param	string	$line
	 */
	function _parse($line)
	{
		$line = trim($line);
		if (strlen($line) > 0)
		{
			if ($line{0} == '#')
			{
				$this->isCommentary = true;
				$line = substr($line, 1);
			}

			$key = $value = '';
			$arr = explode($this->_tab, $line);
			if (count($arr) > 1)
			{
				
				$key = trim($arr[0], '"');
				$value = trim($arr[1], '"');
				if (strlen($key) > 0)
				{
					$this->Key = $key;
					$this->Value = $value;
				}
			}
		}
		else
		{
			$this->isEmptyLine = true;
		}
	}
}