
/*
 * AfterLogic Admin Panel by AfterLogic Corp. <support@afterlogic.com>
 *
 * Copyright (C) 2002-2009  AfterLogic Corp. (www.afterlogic.com)
 * Distributed under the terms of the license described in LICENSE.txt
 * 
 */
 
function CInputSetting (_input)
{
	if (_input == null) {
		this.SetDisable = function () {};
		return;
	}

	this.SetDisable = function (isDisable)
	{
		_input.disabled = isDisable;
		var classArr = _input.className.split(' ');
		if (isDisable) {
			classArr[0] = 'wm_input_disabled';
		}
		else {
			classArr[0] = 'wm_input';
		}
		_input.className = classArr.join(' ');
	}
}

function CDefaultCustomSetting (_defaultRadio, _customRadio, _input, _defaultInputValue)
{
	if (_defaultRadio == null || _customRadio == null || _input == null) {
		this.SetDisable = function () {};
		return;
	}
	
	var _isDefaultMode = true;
	
	this._init = function ()
	{
		var obj = this;
		_defaultRadio.onclick = function () { obj._setMode(true); };
		_customRadio.onclick = function () { obj._setMode(false); };
		this._setMode(_defaultRadio.checked);
	};
	
	this._setMode = function (isDefaultMode)
	{
		_defaultRadio.checked = isDefaultMode;
		_isDefaultMode = isDefaultMode;
		this._setInputDisable(isDefaultMode);
	};
	
	this._setInputDisable = function (isDisable)
	{
		_input.disabled = isDisable;
		var classArr = _input.className.split(' ');
		if (isDisable) {
			classArr[0] = 'wm_input_disabled';
			_input.value = _defaultInputValue;
		}
		else {
			classArr[0] = 'wm_input';
		}
		_input.className = classArr.join(' ');
		
		
	};
	
	this.SetDisable = function (isDisable)
	{
		_defaultRadio.disabled = isDisable;
		_customRadio.disabled = isDisable;
		this._setInputDisable(isDisable || _isDefaultMode);
	};
	
	this._init();
}

function CSwitcher (_checkbox, _setting)
{
	if (_checkbox == null) {
		this.SetDisable = function (isDisable)
		{
			_setting.SetDisable(isDisable);
		};
		return;
	}

	this._onMode = true;
	
	this._init = function ()
	{
		var obj = this;
		_checkbox.onclick = function () { obj._setMode(this.checked); };
		this._setMode(_checkbox.checked);
	};
	
	this._setMode = function (onMode)
	{
		_setting.SetDisable(!onMode)
		this._onMode = onMode;
	};
	
	this.SetDisable = function (isDisable)
	{
		_checkbox.disabled = isDisable;
		_setting.SetDisable(isDisable || !this._onMode);
	};
	
	this._init();
}

var xmSmtpSettings = {
	_constants: {
		SmtpGwConfig: 'NeedTLS=1,OutBind=192.168.1.1',
		DefaultSMTPGateways: '192.168.1.2,192.168.1.15',
		SmtpConfig: 'mail-auth'
	},

	_var_SmtpPort: null,
    _var_SmtpPortSSL: null,

    _box_EnableSMTP_TLS: null,
    _box_EnableAuthSMTP_POP3: null,

	_box_AllowSmtpVRFY: null,
	_smtpPermissions: null,

	_globalGatewayOptions: null,
	_defaultSMTPGatewaysSwitcher: null,
	_smtpAuthSwitcher: null,
	_maxMessageSizeSwitcher: null,
	_maxMessageSizeCheck: null,
	
	Init: function ()
	{
	    this._var_SmtpPort = $('var_SmtpPort');
	    Validator.RegisterAllowNum(this._var_SmtpPort);
	    this._var_SmtpPortSSL = $('var_SmtpPortSSL');
	    Validator.RegisterAllowNum(this._var_SmtpPortSSL);

	    this._box_EnableSMTP_TLS = $('box_EnableSMTP_TLS');
	    this._box_EnableAuthSMTP_POP3 = $('box_EnableAuthSMTP_POP3');
	    
	  	this._box_AllowSmtpVRFY = $('box_AllowSmtpVRFY');
		this._smtpPermissions = $('var_DefaultSmtpPerms');

		var defaultRadio = $('SmtpGlobalGatewayOptionsDefaultRadioID');
	    var customRadio = $('SmtpGlobalGatewayOptionsCustomRadioID');
		var input = $('var_SmtpGwConfig');
		var setting = new CDefaultCustomSetting(defaultRadio, customRadio, input, this._constants.SmtpGwConfig);
	    var checkbox = $('check_SmtpGwConfig');
		this._globalGatewayOptions = new CSwitcher(checkbox, setting);

		defaultRadio = $('DefaultSmtpGatewayListDefaultRadioID');
	    customRadio = $('DefaultSmtpGatewayListCustomRadioID');
		input = $('var_DefaultSMTPGateways');
		Validator.RegisterAllowCommaSeparatedIP(input);
		setting = new CDefaultCustomSetting(defaultRadio, customRadio, input, this._constants.DefaultSMTPGateways);
	    checkbox = $('check_DefaultSMTPGateways');
		this._defaultSMTPGatewaysSwitcher = new CSwitcher(checkbox, setting);
		
		defaultRadio = $('SmtpAuthDefaultRadioID');
	    customRadio = $('SmtpAuthCustomRadioID');
		input = $('var_SmtpConfig');
		setting = new CDefaultCustomSetting(defaultRadio, customRadio, input, this._constants.SmtpConfig);
	    checkbox = $('check_SmtpConfig');
		this._smtpAuthSwitcher = new CSwitcher(checkbox, setting);

		input = $('var_MaxMessageSize');
		Validator.RegisterAllowFloat(input);
		setting = new CInputSetting(input);
	    this._maxMessageSizeCheck = $('check_MaxMessageSize');
		this._maxMessageSizeSwitcher = new CSwitcher(this._maxMessageSizeCheck, setting);
		
		if (null != this._maxMessageSizeCheck) {
	     	this._maxMessageSizeCheck.onchange = function () { obj._updateSmtpPermissions(); };
	    }

		var obj = this;
	    var box_SmtpProto = $('box_SmtpProto');
	    if (null != box_SmtpProto) {
		    box_SmtpProto.onclick = function () { obj._setMode(!this.checked); };
		    this._setMode(!box_SmtpProto.checked);
	    };
	    
	    if (null != this._box_AllowSmtpVRFY) {
	     	this._box_AllowSmtpVRFY.onchange = function () { obj._updateSmtpPermissions(); };
	    };
	    
	    this._updateSmtpPermissions();
	},
	
	_setMode: function (isModeOff)
	{
	    this._var_SmtpPort.readOnly = isModeOff;
	    this._var_SmtpPortSSL.readOnly = isModeOff;

	    if (isModeOff) {
		    this._var_SmtpPort.className = 'wm_input_disabled wm_addmargin';
	    	this._var_SmtpPortSSL.className = 'wm_input_disabled wm_addmargin';
	    } else {
		    this._var_SmtpPort.className = 'wm_input wm_addmargin';
	    	this._var_SmtpPortSSL.className = 'wm_input wm_addmargin';
	    }
	    
	    this._globalGatewayOptions.SetDisable(isModeOff);
	    this._defaultSMTPGatewaysSwitcher.SetDisable(isModeOff);
		this._smtpAuthSwitcher.SetDisable(isModeOff);
		this._maxMessageSizeSwitcher.SetDisable(isModeOff);

	    this._box_EnableSMTP_TLS.disabled = isModeOff;
	    this._box_EnableAuthSMTP_POP3.disabled = isModeOff;
        this._box_AllowSmtpVRFY.disabled = isModeOff;
	},
	
	_updateSmtpPermissions: function ()
	{
		if (null != this._smtpPermissions)
		{
			var _temp = this._smtpPermissions.value;
			if (this._maxMessageSizeCheck.checked) {
				_temp = _temp.replace(/Z/, '');
			} else {
				if(_temp.indexOf('Z') == -1){
					_temp += 'Z';
				};
			};
			
			if (this._box_AllowSmtpVRFY.checked) {
				if (_temp.indexOf('V') == -1) {
					_temp += 'V';
				};
			} else {
				_temp = _temp.replace(/V/, '');
			};
			
			this._smtpPermissions.value = _temp;
		}
	}
}

var xmPop3Settings = {
    _var_Pop3Port: null,
    _var_Pop3PortSSL: null,
    _box_EnablePOP3_TLS: null,
    
	Init: function ()
	{
	    this._var_Pop3Port = $('var_Pop3Port');
	    Validator.RegisterAllowNum(this._var_Pop3Port);
	    this._var_Pop3PortSSL = $('var_Pop3PortSSL');
	    Validator.RegisterAllowNum(this._var_Pop3PortSSL);
	    this._box_EnablePOP3_TLS = $('box_EnablePOP3_TLS');

		var obj = this;
	    var box_Pop3Proto = $('box_Pop3Proto');
	    if (null != box_Pop3Proto) {
		    box_Pop3Proto.onclick = function () { obj._setMode(!this.checked); };
		    this._setMode(!box_Pop3Proto.checked);
	    }
	},
	
	_setMode: function (isModeOff)
	{
	    this._var_Pop3Port.readOnly = isModeOff;
	    this._var_Pop3PortSSL.readOnly = isModeOff;

	    if (isModeOff) {
		    this._var_Pop3Port.className = 'wm_input_disabled wm_addmargin';
	    	this._var_Pop3PortSSL.className = 'wm_input_disabled wm_addmargin';
	    }
	    else {
		    this._var_Pop3Port.className = 'wm_input wm_addmargin';
	    	this._var_Pop3PortSSL.className = 'wm_input wm_addmargin';
	    }
	    
	    this._box_EnablePOP3_TLS.disabled = isModeOff;
	}
}

var xmImap4Settings = {
    _var_Imap4Port: null,
    _var_Imap4PortSSL: null,
    
	Init: function ()
	{
	    this._var_Imap4Port = $('var_Imap4Port');
	    Validator.RegisterAllowNum(this._var_Imap4Port);
	    this._var_Imap4PortSSL = $('var_Imap4PortSSL');
	    Validator.RegisterAllowNum(this._var_Imap4PortSSL);

		var obj = this;
	    var box_Imap4Proto = $('box_Imap4Proto');
	    if (null != box_Imap4Proto) {
		    box_Imap4Proto.onclick = function () { obj._setMode(!this.checked); };
		    this._setMode(!box_Imap4Proto.checked);
	    }
	},
	
	_setMode: function (isModeOff)
	{
	    this._var_Imap4Port.readOnly = isModeOff;
	    this._var_Imap4PortSSL.readOnly = isModeOff;

	    if (isModeOff) {
		    this._var_Imap4Port.className = 'wm_input_disabled wm_addmargin';
	    	this._var_Imap4PortSSL.className = 'wm_input_disabled wm_addmargin';
	    } else {
		    this._var_Imap4Port.className = 'wm_input wm_addmargin';
	    	this._var_Imap4PortSSL.className = 'wm_input wm_addmargin';
	    }
	}
}

var xmDomainsSettings = {
	Init: function ()
	{
		var input = $('var_RootDomain');
		Validator.RegisterAllowDomainSymbols(input);
		
		input = $('var_HeloDomain');
		Validator.RegisterAllowDomainSymbols(input);
		var setting = new CInputSetting(input);
		var checkbox = $('check_HeloDomain');
		var switcher = new CSwitcher(checkbox, setting);
		
		input = $('var_SmtpServerDomain');
		Validator.RegisterAllowDomainSymbols(input);
		setting = new CInputSetting(input);
		checkbox = $('check_SmtpServerDomain');
		switcher = new CSwitcher(checkbox, setting);
		
		input = $('var_POP3Domain');
		Validator.RegisterAllowDomainSymbols(input);
		setting = new CInputSetting(input);
		checkbox = $('check_POP3Domain');
		switcher = new CSwitcher(checkbox, setting);
	}
}

var xmErrorsSettings = {
	_constants: {
		CustMapsList: 'list.dsbl.org.:1,blackholes.mail-abuse.org.:1,dialups.mail-abuse.org.:0'
	},
	
	Init: function ()
	{
		var input = $('var_ErrorsAdmin');
		Validator.RegisterAllowEmailSymbols(input);
		
		input = $('var_TempErrorsAdmin');
		Validator.RegisterAllowEmailSymbols(input);

		input = $('var_NotifyMsgLinesExtra');
		Validator.RegisterAllowNum(input);

		input = $('var_SMTP_MaxErrors');
		Validator.RegisterAllowNum(input);

		input = $('var_CustomSMTPMessage');
		var setting = new CInputSetting(input);
		var checkbox = $('check_CustomSMTPMessage');
		var switcher = new CSwitcher(checkbox, setting);
		
		input = $('var_NotifyTryPattern');
		Validator.RegisterAllowCommaSeparatedNum(input);
		setting = new CInputSetting(input);
		checkbox = $('check_NotifyTryPattern');
		switcher = new CSwitcher(checkbox, setting);
		
		input = $('var_Pop3SyncErrorAccount');
		Validator.RegisterAllowEmailSymbols(input);
		setting = new CInputSetting(input);
		checkbox = $('check_Pop3SyncErrorAccount');
		switcher = new CSwitcher(checkbox, setting);

		var defaultRadio = $('CustMapsListDefaultRadioID');
	    var customRadio = $('CustMapsListCustomRadioID');
		input = $('var_CustMapsList');
		setting = new CDefaultCustomSetting(defaultRadio, customRadio, input, this._constants.CustMapsList);
	    checkbox = $('check_CustMapsList');
		switcher = new CSwitcher(checkbox, setting);
	}
}

var xmMiscSettings = {
	_constants: {
		SmartDNSHost: 'dns.home.bogus.net:tcp,192.168.1.1:udp',
		FetchHdrTags: '+X-Deliver-To,+Received,To,Cc'
	},
	
	Init: function ()
	{
		var input = $('var_FingerPort');
		Validator.RegisterAllowNum(input);
		var setting = new CInputSetting(input);
		var checkbox = $('box_FingerProto');
		var switcher = new CSwitcher(checkbox, setting);
		
		input = $('var_MaxMTAOps');
		Validator.RegisterAllowNum(input);
		
		input = $('var_SmtpMinDiskSpace');
		Validator.RegisterAllowFloat(input);
		setting = new CInputSetting(input);
		checkbox = $('check_SmtpMinDiskSpace');
		switcher = new CSwitcher(checkbox, setting);
		
		input = $('var_SmtpMinVirtMemSpace');
		Validator.RegisterAllowFloat(input);
		setting = new CInputSetting(input);
		checkbox = $('check_SmtpMinVirtMemSpace');
		switcher = new CSwitcher(checkbox, setting);

		input = $('var_Pop3MinVirtMemSpace');
		Validator.RegisterAllowNum(input);
		setting = new CInputSetting(input);
		checkbox = $('check_Pop3MinVirtMemSpace');
		switcher = new CSwitcher(checkbox, setting);

		input = $('var_ReceivedHdrType');
		setting = new CInputSetting(input);
		checkbox = $('check_ReceivedHdrType');
		switcher = new CSwitcher(checkbox, setting);

		var defaultRadio = $('SmartDnsHostDefaultRadioID');
	    var customRadio = $('SmartDnsHostCustomRadioID');
		input = $('var_SmartDNSHost');
		setting = new CDefaultCustomSetting(defaultRadio, customRadio, input, this._constants.SmartDNSHost);
	}
}

var xmLogSettings = {
    _box_LogPOP3: null,
    _box_Pop3LogPasswd: null,
    
	Init: function ()
	{
	    var box_LogPOP3 = $('box_LogPOP3');
	    this._box_Pop3LogPasswd = $('box_Pop3LogPasswd');

		var obj = this;
	    if (null != box_LogPOP3) {
		    box_LogPOP3.onclick = function () { obj._setMode(this.checked); };
	    }
	},
	
	_setMode: function (pop3LogChecked)
	{
	    this._box_Pop3LogPasswd.checked = pop3LogChecked;
	}
}

var xmAdminSettings = {
    _var_ControlPort: null,
    _var_ControlPortSSL: null,
    _box_EnableCTRL_TLS: null,
    
	Init: function ()
	{
		var input = $('var_PostMaster');
		Validator.RegisterAllowEmailSymbols(input);
		
	    this._var_ControlPort = $('var_ControlPort');
	    Validator.RegisterAllowNum(this._var_ControlPort);
	    this._var_ControlPortSSL = $('var_ControlPortSSL');
	    Validator.RegisterAllowNum(this._var_ControlPortSSL);
	    this._box_EnableCTRL_TLS = $('box_EnableCTRL_TLS');
	    
	    var obj = this;
	    
	    var box_ControlProto = $('box_ControlProto');
	    if (null != box_ControlProto) {
		    box_ControlProto.onclick = function () { return obj._setMode(!this.checked, true); };
		    this._setMode(!box_ControlProto.checked, false);
	    }
	    
		var box_TlsControlProto = $('box_EnableCTRL_TLS');
		var box_TlsControlPort = $('var_ControlPortSSL');
	    if (null != box_TlsControlProto && null != box_TlsControlPort) {
		    box_TlsControlProto.onclick = function () { SetDisabled(box_TlsControlPort, !(box_TlsControlProto.checked))	};
		    SetDisabled(box_TlsControlPort, !(box_TlsControlProto.checked));
	    }
	},

	_setMode: function (isModeOff, needConfirm)
	{
		var confirmMsg = 'Are you sure you want to disable AfterLogic XMail Server Remote Control? If you disable this, you\'ll be not able to CONFIGURE YOUR AfterLogic XMail Server THROUGH WEB INTERFACE.';
		if (isModeOff && needConfirm && !confirm(confirmMsg)) return false;
		
	    this._var_ControlPort.readOnly = isModeOff;
	    this._var_ControlPortSSL.readOnly = isModeOff;

	    if (isModeOff) {
		    this._var_ControlPort.className = 'wm_input_disabled wm_addmargin';
	    	this._var_ControlPortSSL.className = 'wm_input_disabled wm_addmargin';
	    }
	    else {
		    this._var_ControlPort.className = 'wm_input wm_addmargin';
	    	this._var_ControlPortSSL.className = 'wm_input wm_addmargin';
	    }
	    
	    this._box_EnableCTRL_TLS.disabled = isModeOff;
	    
	    return true;
	}
}

var SettingsObjects = Array();
SettingsObjects['smtp'] = xmSmtpSettings;
SettingsObjects['pop3'] = xmPop3Settings;
SettingsObjects['imap4'] = xmImap4Settings;
SettingsObjects['domains'] = xmDomainsSettings;
SettingsObjects['errors'] = xmErrorsSettings;
SettingsObjects['misc'] = xmMiscSettings;
SettingsObjects['log'] = xmLogSettings;
SettingsObjects['admin'] = xmAdminSettings;